import pytest
import json
from .models import Data
from toardb.timeseries.models import Timeseries
from toardb.stationmeta.models import StationmetaCore
from toardb.stationmeta.schemas import get_geom_from_coordinates, Coordinates
from toardb.variables.models import Variable
from toardb.contacts.models import Person, Organisation
from toardb.auth_user.models import AuthUser
from toardb.test_base import (
    client,
    get_test_db,
    create_test_database,
    url,
    get_test_engine,
    test_db_session as db,
)

class TestApps:
    def setup(self):
        self.application_url = "/data/"

    """Set up all the data before each test
       If you want the setup only once (per test module),
       the scope argument is not working in the expected way, as discussed here:
       https://stackoverflow.com/questions/45817153/py-test-fixture-use-function-fixture-in-scope-fixture
    """
    @pytest.fixture(autouse=True)
    def setup_db_data(self, db):
        _db_conn = get_test_engine()
        fake_conn = _db_conn.raw_connection()
        fake_cur = fake_conn.cursor()
        # id_seq will not be reset automatically between tests!
        fake_cur.execute("ALTER SEQUENCE auth_user_id_seq RESTART WITH 1")
        fake_conn.commit()
        fake_cur.execute("ALTER SEQUENCE variables_id_seq RESTART WITH 1")
        fake_conn.commit()
        fake_cur.execute("ALTER SEQUENCE stationmeta_core_id_seq RESTART WITH 1")
        fake_conn.commit()
        fake_cur.execute("ALTER SEQUENCE stationmeta_global_id_seq RESTART WITH 1")
        fake_conn.commit()
        fake_cur.execute("ALTER SEQUENCE stationmeta_global_services_id_seq RESTART WITH 1")
        fake_conn.commit()
        fake_cur.execute("ALTER SEQUENCE stationmeta_roles_id_seq RESTART WITH 1")
        fake_conn.commit()
        fake_cur.execute("ALTER SEQUENCE stationmeta_annotations_id_seq RESTART WITH 1")
        fake_conn.commit()
        fake_cur.execute("ALTER SEQUENCE stationmeta_aux_doc_id_seq RESTART WITH 1")
        fake_conn.commit()
        fake_cur.execute("ALTER SEQUENCE stationmeta_aux_image_id_seq RESTART WITH 1")
        fake_conn.commit()
        fake_cur.execute("ALTER SEQUENCE stationmeta_aux_url_id_seq RESTART WITH 1")
        fake_conn.commit()
        fake_cur.execute("ALTER SEQUENCE persons_id_seq RESTART WITH 1")
        fake_conn.commit()
        fake_cur.execute("ALTER SEQUENCE organisations_id_seq RESTART WITH 1")
        fake_conn.commit()
        fake_cur.execute("ALTER SEQUENCE timeseries_annotations_id_seq RESTART WITH 1")
        fake_conn.commit()
        fake_cur.execute("ALTER SEQUENCE timeseries_roles_id_seq RESTART WITH 1")
        fake_conn.commit()
        fake_cur.execute("ALTER SEQUENCE timeseries_id_seq RESTART WITH 1")
        fake_conn.commit()
        infilename = "toardb/auth_user/fixtures/auth.json"
        with open(infilename) as f:
            metajson=json.load(f)
            for entry in metajson:
                new_auth_user = AuthUser(**entry)
                db.add(new_auth_user)
                db.commit()
                db.refresh(new_auth_user)
        infilename = "toardb/contacts/fixtures/persons.json"
        with open(infilename) as f:
            metajson=json.load(f)
            for entry in metajson:
                new_person = Person(**entry)
                db.add(new_person)
                db.commit()
                db.refresh(new_person)
        infilename = "toardb/contacts/fixtures/organisations.json"
        with open(infilename) as f:
            metajson=json.load(f)
            for entry in metajson:
                new_organisation = Organisation(**entry)
                db.add(new_organisation)
                db.commit()
                db.refresh(new_organisation)
        infilename = "toardb/variables/fixtures/variables.json"
        with open(infilename) as f:
            metajson=json.load(f)
            for entry in metajson:
                new_variable = Variable(**entry)
                db.add(new_variable)
                db.commit()
                db.refresh(new_variable)
        infilename = "toardb/stationmeta/fixtures/stationmeta_core.json"
        with open(infilename) as f:
            metajson=json.load(f)
            for entry in metajson:
                new_stationmeta_core = StationmetaCore(**entry)
                # there's a mismatch with coordinates --> how to automatically switch back and forth?!
                new_stationmeta_core.coordinates = get_geom_from_coordinates(Coordinates(**new_stationmeta_core.coordinates))
                # there's also a mismatch with additional_metadata --> BUT: this should not be switched back!
                # in upload command, we have now: "additional_metadata": "{}"
                # but return from this method gives: "additional_metadata": {}
                # ==> there is a mismatch between model(JSONB) and schema(JSON)
                new_stationmeta_core.additional_metadata = str(new_stationmeta_core.additional_metadata)
                db.add(new_stationmeta_core)
                db.commit()
                db.refresh(new_stationmeta_core)
        infilename = "toardb/timeseries/fixtures/timeseries.json"
        with open(infilename) as f:
            metajson=json.load(f)
            for entry in metajson:
                new_timeseries = Timeseries(**entry)
                db.add(new_timeseries)
                db.commit()
                db.refresh(new_timeseries)
        infilename = "toardb/data/fixtures/data.json"
        with open(infilename) as f:
            metajson=json.load(f)
            for entry in metajson:
                new_data = Data (**entry)
                db.add(new_data)
                db.commit()
                db.refresh(new_data)


    def test_get_data(self, client, db):
        response = client.get("/data/?limit=4")
        expected_status_code = 200
        assert response.status_code == expected_status_code
        expected_resp = [{'datetime': '2012-12-16T21:00:00+01:00', 'value': 21.581, 'flags': 0, 'timeseries_id': 1},
                         {'datetime': '2012-12-16T22:00:00+01:00', 'value': 13.734, 'flags': 0, 'timeseries_id': 1},
                         {'datetime': '2012-12-16T23:00:00+01:00', 'value': 13.734, 'flags': 0, 'timeseries_id': 1},
                         {'datetime': '2012-12-17T00:00:00+01:00', 'value':  7.848, 'flags': 0, 'timeseries_id': 1}]
        assert response.json() == expected_resp


    def test_get_special(self, client, db):
        response = client.get("/data/1")
        expected_status_code = 200
        assert response.status_code == expected_status_code
        expected_resp = [{'datetime': '2012-12-16T21:00:00+01:00', 'value': 21.581, 'flags': 0, 'timeseries_id': 1},
                         {'datetime': '2012-12-16T22:00:00+01:00', 'value': 13.734, 'flags': 0, 'timeseries_id': 1},
                         {'datetime': '2012-12-16T23:00:00+01:00', 'value': 13.734, 'flags': 0, 'timeseries_id': 1},
                         {'datetime': '2012-12-17T00:00:00+01:00', 'value':  7.848, 'flags': 0, 'timeseries_id': 1},
                         {'datetime': '2012-12-17T01:00:00+01:00', 'value': 15.696, 'flags': 0, 'timeseries_id': 1},
                         {'datetime': '2012-12-17T02:00:00+01:00', 'value': 11.772, 'flags': 0, 'timeseries_id': 1},
                         {'datetime': '2012-12-17T03:00:00+01:00', 'value': 13.734, 'flags': 0, 'timeseries_id': 1},
                         {'datetime': '2012-12-17T04:00:00+01:00', 'value': 19.62,  'flags': 0, 'timeseries_id': 1},
                         {'datetime': '2012-12-17T05:00:00+01:00', 'value': 15.696, 'flags': 0, 'timeseries_id': 1},
                         {'datetime': '2012-12-17T06:00:00+01:00', 'value':  5.886, 'flags': 0, 'timeseries_id': 1}]
        assert response.json() == expected_resp


#   def test_insert_new_without_credits(self):
#?      response = client.post("/data/")
#       expected_status_code=401
#       assert response.status_code == expected_status_code
#?      expected_resp = ...
#   assert response.json() == expected_resp



#   def test_insert_new_wrong_credits(self):
#?      response = client.post("/data/")
#       expected_status_code = 401
#       assert response.status_code == expected_status_code
#?      expected_resp = ...
#   assert response.json() == expected_resp


    def test_insert_new(self, client, db):
        response = client.post("/data/", files={"file": open("toardb/data/fixtures/toluene_SDZ54421_2013_2013_v1-0.dat", "rb")})
        expected_status_code = 200
        assert response.status_code == expected_status_code
        expected_resp = 'Data successfully inserted.'
        assert response.json() == expected_resp

                                                    
    def test_insert_duplicate(self, client, db):
        response = client.post("/data/", files={"file": open("toardb/data/fixtures/toluene_SDZ54421_2012_2012_v1-0.dat", "rb")})
        expected_status_code = 400
        assert response.status_code == expected_status_code
        expected_resp = {'detail': 'Data for timeseries already registered.'}
        assert response.json() == expected_resp

                                                    
    def test_insert_missing_id(self, client, db):
        response = client.post("/data/", files={"file": open("toardb/data/fixtures/o3_SDZ54421_2012_2012_v1-0.dat", "rb")})
        expected_status_code = 400
        assert response.status_code == expected_status_code
        print (response.json())
        expected_resp = {'detail': '"Timeseries not found for station SDZ54421, variable o3, label CMA"'}
        assert response.json() == expected_resp
